<?php

namespace Controller;

use Exception;
use Traits\TResponse;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Service\WorkItem as WorkItemService;

class WorkItem extends Simple
{

    use TResponse;

    /**
     * @var WorkItemService
     */
    private $workItemService;

    /**
     * WorkItem constructor.
     * @param WorkItemService
     */
    public function __construct(WorkItemService $workItemService)
    {
        $this->workItemService = $workItemService;
    }

    public function findWorkItem(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $data = $this->workItemService->findWorkItem($params);
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findLevelPattern(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->workItemService->findLevelPattern();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findMasterItemSelectItems(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->workItemService->findMasterItemSelectItems();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findWorkItemType(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->workItemService->findWorkItemType();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insert(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $this->workItemService->insert($params);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function update(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $this->workItemService->update($params);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    public function findItemtrab(DTO\Request $request, DTO\Response $response) {
        try {
            $result = $this->workItemService->findItemtrab();
            $this->sendResponse($result, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
